#pragma once

#include <QtCore/QSharedPointer>
#include <QtCore/QWeakPointer>
#include <QtCore/QUrl>
#include <QtCore/QObject>

#include <odb/database.hxx>

#pragma db object abstract
class DbObject: public QObject {
  Q_OBJECT
public:
  typedef unsigned long long Id;
  static const Id ID_NEW;

  #pragma db id auto
  Id id;

protected:
  DbObject(): id(ID_NEW) {}
  virtual ~DbObject() {}
};


#pragma db object polymorphic
class Document : public DbObject {
  Q_OBJECT
public:
  typedef QSharedPointer<Document> Ptr;
  static Ptr load(odb::database& db);
  virtual void save(odb::database& db);

  friend class odb::access;
};

/**
* TRIVIAL CLASS WITHOUT SIMPLE MEMBERS RESULTS IN BAD SQL QUERY
*/
#pragma db object polymorphic
class Workflow : public Document {
  Q_OBJECT
public:

  //QString dummy;            // UNCOMMENT THIS LINE TO PASS THE TEST ;)
  QList<QString> data;        // CONTAINER TYPES ARE STORED IN SEPARATE SQL TABLES

  friend class odb::access;
};
