#include "Document.hpp"

#include <odb/sqlite/database.hxx>
#include <odb/schema-catalog.hxx>
#include <odb/statement.hxx>

#include <QtCore/QFile>
#include <QtCore/QDir>
#include <QtCore/QDebug>

const QString testfile = "collections_fail";

odb::database* getDb(const QString& filename) {
  const QByteArray convertedFileName = QString("%1/%2.db").arg(QDir::current().path(), filename).toUtf8();
  qDebug() << "  Opening DB" << convertedFileName;
  odb::database* result = new odb::sqlite::database(convertedFileName.data(), SQLITE_OPEN_CREATE | SQLITE_OPEN_READWRITE);
// enable/disable trace
#if 0
  result->tracer(odb::stderr_tracer);
#endif
  // check if database file contains tables, if so - consider that scheme is already created, otherwise - create new one
  odb::transaction t(result->begin());
  const unsigned long long numberOfTables = result->execute("SELECT name FROM sqlite_master WHERE type='table';");
  t.commit();
  if (numberOfTables == 0) {
    // create the database schema;
    // due to bugs in SQLite foreign key support for DDL statements, we need to temporarily disable foreign keys
    odb::connection_ptr c(result->connection());
    c->execute("PRAGMA foreign_keys=OFF");
    odb::transaction t(c->begin());
    odb::schema_catalog::create_schema(*result, "Engine");
    t.commit();
    c->execute("PRAGMA foreign_keys=ON");
  }
  return result;
}

void save() {
	qDebug() << "Saving...";
	odb::database* const databaseObj = getDb(testfile);

	QFile::remove(testfile);

	Workflow p;
	p.data << "TEST";
	p.save(*databaseObj);

	delete databaseObj;
}

void load() {
	qDebug() << "Loading...";
	odb::database* const databaseObj = getDb(testfile);

	Document::Ptr p = Document::load(*databaseObj);
	Workflow* pw = (Workflow*)p.data();

	qDebug() << pw->data << "Passed";
	Q_ASSERT(pw->data.size() == 1);
	Q_ASSERT(pw->data[0]  == "TEST");

	delete databaseObj;
}

int main(int argc, char *argv[]) {
	save();
	load();
}
