#include "Document.hpp"

#include <odb/sqlite/database.hxx>
#include <odb/schema-catalog.hxx>
#include <odb/statement.hxx>

#include <QtCore/QFile>
#include <QtCore/QDir>
#include <QtCore/QDebug>

const QString testfile = "traits_fail";

odb::database* getDb(const QString& filename) {
  const QByteArray convertedFileName = QString("%1/%2.db").arg(QDir::current().path(), filename).toUtf8();
  qDebug() << "  Opening DB" << convertedFileName;
  odb::database* result = new odb::sqlite::database(convertedFileName.data(), SQLITE_OPEN_CREATE | SQLITE_OPEN_READWRITE);
// enable/disable trace
#if 0
  result->tracer(odb::stderr_tracer);
#endif
  // check if database file contains tables, if so - consider that scheme is already created, otherwise - create new one
  odb::transaction t(result->begin());
  const unsigned long long numberOfTables = result->execute("SELECT name FROM sqlite_master WHERE type='table';");
  t.commit();
  if (numberOfTables == 0) {
    // create the database schema;
    // due to bugs in SQLite foreign key support for DDL statements, we need to temporarily disable foreign keys
    odb::connection_ptr c(result->connection());
    c->execute("PRAGMA foreign_keys=OFF");
    odb::transaction t(c->begin());
    odb::schema_catalog::create_schema(*result, "Engine");
    t.commit();
    c->execute("PRAGMA foreign_keys=ON");
  }
  return result;
}

void save(const bool value) {
	qDebug() << "Saving...";
	odb::database* const databaseObj = getDb(testfile);

	QFile::remove(testfile);

	Document p;
	p.data = KludgyBool(value);
	p.save(*databaseObj);

	delete databaseObj;
}

void load(const bool value) {
	qDebug() << "Loading...";
	odb::database* const databaseObj = getDb(testfile);

	Document::Ptr p = Document::load(*databaseObj);

	Q_ASSERT(p->data.value == value);
	qDebug() << p->data.value << "Passed";

	delete databaseObj;
}

int main(int argc, char *argv[]) {
	save(true);
	load(true);
	save(false);
	load(false);
}
